/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 * @defgroup pwm_example_main main.c
 * @{
 * @ingroup pwm_example
 * 
 * @brief  PWM Example Application main file.
 *
 * This file contains the source code for a sample application using PWM.
 *
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"
#include "app_error.h"
#include "bsp.h"
#include "nrf_delay.h"
#include "app_pwm.h"
#include "app_button.h"
#include "boards.h"
#include "nrf_gpio.h"
#include "app_timer.h"
#include "nrf_drv_gpiote.h"
#define APP_TIMER_PRESCALER             0                                           // Value of the RTC1 PRESCALER register.
//#define APP_TIMER_MAX_TIMERS          2                                           // Maximum number of simultaneously created timers. // NORDIC One for debounce, one for long press detect
#define APP_TIMER_OP_QUEUE_SIZE         2                                           // Size of timer operation queues. 

#define BUTTON_DEBOUNCE_DELAY		        50                                          // Delay from a GPIOTE event until a button is reported as pushed. 
#define LONG_BUTTON_PRESS_TIME_MS       5000
#define BUTTON_PRESS_TIME_MS            3000
//#define INVALID_PIN      0XFF
#define MS_TO_TICKS(ms)                  (ms*32768/1000)

APP_TIMER_DEF(m_timer_3_sec_id);
APP_TIMER_DEF(m_timer_2_sec_id);


#define INITIAL_LLS_ALERT_LEVEL             BLE_CHAR_ALERT_LEVEL_NO_ALERT                /**< Initial value for the Alert Level characteristic in the Link Loss service. */
#define TX_POWER_LEVEL                      (-8)                                         /**< TX Power Level value. This will be set both in the TX Power service, in the advertising data, and also used to set the radio transmit power. */

#define PWM_OUTPUT1 6
#define PWM_OUTPUT2 10

//static uint8_t m_button_action_pin = INVALID_PIN;                                 /**< Pin number for button that had recent activity. */

#define MAX_TEST_DATA_BYTES     (15U)                                               /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE 256                                                        /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 1                                                          /**< UART RX buffer size. */
#define DEAD_BEEF                           0xDEADBEEF                                   /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

APP_PWM_INSTANCE(PWM0,1);                                                           // Create the instance "PWM0" using TIMER2.



																		
/**@brief Callback function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num   Line number of the failing ASSERT call.
 * @param[in] file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


static void pwm_uninit(void)
{
     			app_pwm_disable(&PWM0);
		  nrf_drv_gpiote_out_task_disable(6);
      //nrf_drv_gpiote_out_task_disable(10);
			nrf_gpio_pin_clear(6); //clear the LED                              
      nrf_gpio_pin_clear(10);//clear the LED
			 //nrf_gpio_pin_set(10);//clear the LED
			app_pwm_uninit(&PWM0);
}


static void pwm_init(void)
{
	
	  nrf_gpio_pin_set(10);//clear the LED
	  nrf_gpio_pin_set(6);//clear the LED
	  //printf" PWM Initialization:\n");
	  ret_code_t err_code;
	  
			/* 2-channel PWM */
			app_pwm_config_t pwm0_cfg = APP_PWM_DEFAULT_CONFIG_2CH(5000UL,6,10);
			
			/* Switch the polarity of the second channel. */
			pwm0_cfg.pin_polarity[0] = APP_PWM_POLARITY_ACTIVE_HIGH;
			
			/* Initialize and enable PWM. */
			err_code = app_pwm_init(&PWM0,&pwm0_cfg,NULL);
			APP_ERROR_CHECK(err_code);
			
			app_pwm_enable(&PWM0);
			app_pwm_channel_duty_set(&PWM0, 0, 50);
 
}

/*
 * Handler to be called on single shot timer event.
 */
static void timer_handler_3_sec(void * p_context)
{
    if(!nrf_gpio_pin_read(BUTTON_1))
    {
        app_timer_start(m_timer_2_sec_id, MS_TO_TICKS(2000), NULL); // NORDIC: Start 2 second single shot timer
    }
		
    
}
/*
 * Handler to be called on single shot timer event.
 */
static void timer_handler_2_sec(void * p_context)
{
   // now 3+2 seconds have expired, check the button state
    if(!nrf_gpio_pin_read(BUTTON_1))
    {
	      pwm_uninit();
	      //printf"Button is pressed for 5 seconds:\n");
        nrf_gpio_pin_set(LED_3); //clear the LED     
    }
    else
    { 
            // side effect of using same button to decide two things is that your pwm will start two seconds late.
        	      pwm_init();
	      //printf"Button is pressed for 3 seconds:\n");
        nrf_gpio_pin_clear(LED_3); //clear the LED  
    }
}
/*
 * Handler to be called when button is pushed.
 * param[in]   pin_no 			The pin number where the event is genereated
 * param[in]   button_action 	Is the button pushed or released
 */
static void button_handler(uint8_t pin_no, uint8_t button_action)
{
    if(button_action == APP_BUTTON_PUSH)
    {
        switch(pin_no)
        {			
            case BUTTON_1:				
                app_timer_start(m_timer_3_sec_id, MS_TO_TICKS(3000), NULL); // NORDIC: Start 3 second single shot timer
                break;
            default:
                break;
        }
    }
    else if(button_action == APP_BUTTON_RELEASE)
    {
        // Stop all timers if button is released
        app_timer_stop(m_timer_3_sec_id);
    }                              
    
		
}
/**  
 * Initialize the clock.
 */
void init_clock()
{
	
	  //printf"clock Initialization:\n");
    NRF_CLOCK->LFCLKSRC            = (CLOCK_LFCLKSRC_SRC_Xtal << CLOCK_LFCLKSRC_SRC_Pos);
    NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_LFCLKSTART    = 1;
    while (NRF_CLOCK->EVENTS_LFCLKSTARTED == 0); // Wait for clock to start
}	

/**
 * Initialize all four LEDs on the nRF51 DK.
 */
void init_leds()
{   //printf"LEDS Initialization:\n");
    nrf_gpio_range_cfg_input(BUTTON_1, BUTTON_2, NRF_GPIO_PIN_PULLUP);//configured button with internal pullup resistor
    nrf_gpio_range_cfg_output(LED_1, LED_4); 
	  nrf_gpio_pin_set(LED_1);
    nrf_gpio_pin_set(LED_2);
	  nrf_gpio_pin_set(LED_3); //clear the LED                              
    nrf_gpio_pin_set(LED_4);//clear the LED
	  
                                                                                                                                  
}	

/**@brief Function for initializing the button handler module.
 */
static void buttons_init(void)
{
	   //printf"Button Initialization:\n");
     uint32_t err_code;
    // Button configuration structure.
    static app_button_cfg_t p_button[] = {  {BUTTON_1, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler},
                                          {BUTTON_2, APP_BUTTON_ACTIVE_LOW, NRF_GPIO_PIN_PULLUP, button_handler}};
        
    // Macro for initializing the application timer module.
    // It will handle dimensioning and allocation of the memory buffer required by the timer, making sure that the buffer is correctly aligned. It will also connect the timer module to the scheduler (if specified).
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, NULL);    
    app_timer_create(&m_timer_3_sec_id, APP_TIMER_MODE_SINGLE_SHOT, timer_handler_3_sec); // NORDIC Create single shot timer triggering interrupt handler timer_handler_3_sec once
    app_timer_create(&m_timer_2_sec_id, APP_TIMER_MODE_SINGLE_SHOT, timer_handler_2_sec); // NORDIC Create single shot timer triggering interrupt handler timer_handler_3_sec once
    
    // Initializing the buttons.
    err_code = app_button_init(p_button, sizeof(p_button) / sizeof(p_button[0]), BUTTON_DEBOUNCE_DELAY);
    APP_ERROR_CHECK(err_code);
                                            
    // Enabling the buttons.										
    err_code = app_button_enable();
    APP_ERROR_CHECK(err_code);

}


int main(void)
	
{ 

	 
	  // Initialize.

	  
		init_leds();
    init_clock();
		buttons_init();  
    
    while(1)
    {
        // maybe you app needs low power mode?
        __WFE();
    };
		
		
}


/** @} */
